"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelMiddleware = void 0;
const cm_libs_utils_1 = require("cm-libs-utils");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
var RecordStatus;
(function (RecordStatus) {
    RecordStatus["ACTIVE"] = "ACTIVE";
    RecordStatus["INACTIVE"] = "INACTIVE";
})(RecordStatus || (RecordStatus = {}));
class ModelMiddleware {
    constructor(client, model, accessInfo, errorHandler) {
        if (!(0 <= accessInfo?.requestedBy)) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedBy parameter is missing', { accessInfo });
        }
        accessInfo?.requestedByName || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedByName parameter is missing', { accessInfo });
        client.$use(async (params, next) => {
            const start = (0, cm_libs_utils_1.timeNow)();
            switch (params.action) {
                case 'create':
                case 'createMany':
                    params = this.packCreateArgs(params);
                    break;
                case 'update':
                case 'updateMany':
                    params = this.packUpdateArgs(params);
            }
            try {
                const result = await next(params);
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `Run ${params.model}.${params.action} affected ${affectedCount(params.action, result)} records took ${(0, cm_libs_utils_1.timeElapsed)(start)} ms`);
                return result;
            }
            catch (error) {
                errorHandler ? errorHandler(error, params) : (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.DATABASE_ERROR, params);
            }
        });
        this.model = client[model];
        this.accessInfo = accessInfo;
    }
    packCreateArgs(params) {
        const requestedBy = this.accessInfo.requestedBy;
        const requestedByName = this.accessInfo.requestedByName;
        if (!Array.isArray(params.args.data)) {
            params.args.data = {
                ...params.args.data,
                createdBy: requestedBy,
                createdByName: requestedByName,
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
            return params;
        }
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...params.args.data[i],
                createdBy: requestedBy,
                createdByName: requestedByName,
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
        }
        return params;
    }
    packUpdateArgs(params) {
        const requestedBy = this.accessInfo.requestedBy;
        const requestedByName = this.accessInfo.requestedByName;
        if (!Array.isArray(params.args.data)) {
            params.args.data = {
                ...params.args.data,
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
            return params;
        }
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...params.args.data[i],
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
        }
        return params;
    }
    async create(params) {
        return this.model['create'](params);
    }
    async createMany(params) {
        return this.model['createMany'](params);
    }
    async update(params) {
        return this.model['update'](params);
    }
    async updateMany(params) {
        return this.model['updateMany'](params);
    }
    async browse(params) {
        const pagination = params.pagination || {};
        const options = params.options || {};
        delete params.pagination;
        delete params.options;
        pagination.page = !(0 < pagination.page) ? 1 : pagination.page;
        pagination.pageSize ??= 10;
        params.where ??= {
            status: RecordStatus.ACTIVE,
            ...params.where
        };
        params.orderBy ??= { createdAt: 'desc' };
        if (0 < pagination.pageSize) {
            params.take = pagination.pageSize;
            params.skip = (pagination.page - 1) * pagination.pageSize;
        }
        const promises = [this.model['findMany'](params)];
        if (!(options.speedMode ??= false)) {
            promises.push(this.model['count']({ where: params.where }));
        }
        const results = await Promise.all(promises);
        const properties = options.speedMode ? pagination : { ...pagination, count: results[1] };
        return {
            items: results[0],
            properties
        };
    }
}
exports.ModelMiddleware = ModelMiddleware;
function affectedCount(action, result) {
    let count;
    switch (action) {
        case 'findMany':
            count = result?.length;
            break;
        case 'count':
            count = result;
            break;
        case 'create':
        case 'update':
            count = 1;
            break;
        case 'createMany':
        case 'updateMany':
            count = result?.count;
    }
    return count || 0;
}
//# sourceMappingURL=model-middleware.js.map