"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModel = void 0;
const lodash_1 = __importDefault(require("lodash"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
var RecordStatus;
(function (RecordStatus) {
    RecordStatus["ACTIVE"] = "ACTIVE";
    RecordStatus["INACTIVE"] = "INACTIVE";
})(RecordStatus || (RecordStatus = {}));
class BaseModel {
    constructor(prisma, formatErrorHandler, modelName) {
        this.prisma = prisma;
        this.modelName = modelName;
        this.formatErrorHandler = formatErrorHandler;
    }
    async add(input, options) {
        try {
            const requestedBy = options?.accessInfo?.requestedBy || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedBy parameter is missing', { options });
            const requestedByName = options?.accessInfo?.requestedByName || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedByName parameter is missing', { options });
            const now = new Date(new Date().toISOString());
            const data = {
                createdBy: requestedBy,
                createdByName: requestedByName,
                createdAt: now,
                updatedBy: requestedBy,
                updatedByName: requestedByName,
                updatedAt: now,
                ...input
            };
            return await this.prisma.create({ data: data });
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'add', [input, options]);
        }
    }
    /* eslint-disable @typescript-eslint/no-unused-vars */
    async read(where, options) {
        try {
            return await this.prisma.findUnique({ where });
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'read', [where]);
        }
    }
    async browse(pagination, where, options) {
        options = lodash_1.default.cloneDeep(options);
        try {
            pagination ??= {};
            pagination.page ??= 1;
            pagination.pageSize ??= 10;
            where = {
                status: RecordStatus.ACTIVE,
                ...where
            };
            const skip = (pagination.page - 1) * pagination.pageSize;
            options.scanAll ??= false;
            options.scanAll = pagination.pageSize === 0 ? true : false;
            options.sort ??= { createdAt: 'asc' };
            options.speedMode ??= false;
            let count = 0;
            if (!options.speedMode) {
                count = await this.prisma.count({ where });
            }
            let configuration = { orderBy: [options.sort], where };
            if (!options.scanAll) {
                configuration = { ...configuration, take: pagination.pageSize, skip };
            }
            return {
                items: await this.prisma.findMany(configuration),
                properties: options.speedMode ? { ...pagination } : { ...pagination, count }
            };
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'browse', [pagination, where, options]);
        }
    }
    async edit(input, where, options) {
        try {
            const requestedBy = options?.accessInfo?.requestedBy || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedBy parameter is missing', { options });
            const requestedByName = options?.accessInfo?.requestedByName || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedByName parameter is missing', { options });
            const data = { ...input, updatedBy: requestedBy, updatedByName: requestedByName };
            delete input['id'];
            return await this.prisma.update({ where, data });
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'edit', [input, where, options]);
        }
    }
    async delete(where, options) {
        try {
            return await this.prisma.delete({ where });
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'delete', [where, options]);
        }
    }
    async activate(where, options) {
        try {
            const requestedBy = options?.accessInfo?.requestedBy || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedBy parameter is missing', { options });
            const requestedByName = options?.accessInfo?.requestedByName || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedByName parameter is missing', { options });
            const data = {
                status: RecordStatus.ACTIVE,
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
            return await this.edit(data, where, options);
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'activate', [where, options]);
        }
    }
    async deactivate(where, options) {
        try {
            const requestedBy = options?.accessInfo?.requestedBy || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedBy parameter is missing', { options });
            const requestedByName = options?.accessInfo?.requestedByName || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedByName parameter is missing', { options });
            const data = {
                status: RecordStatus.INACTIVE,
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
            return await this.edit(data, where, options);
        }
        catch (error) {
            this.formatErrorHandler(error, this.modelName, 'deactivate', [where, options]);
        }
    }
}
exports.BaseModel = BaseModel;
//# sourceMappingURL=model.js.map